//+------------------------------------------------------------------+
//|                                        BasketTR_OrderInclude.mqh |
//|                                             2009.06.04 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2009.06.04 SwingMan"
#property link      ""

/* PROCEDURES:
void Get_ActivOrder()
void Get_LotsNumber()
void SendOrder_BuyMarket(double nLots, int iMagicNumber, string stComment)
void SendOrder_SellMarket(double nLots, int iMagicNumber, string stComment)
bool Modify_LongStopLoss(int iMagicNumber, double NewStopLoss)
bool Modify_ShortStopLoss(int iMagicNumber, double NewStopLoss)
void Send_ErrorDescription(string sSymbol, string sText)
bool SendOrders_IsAllowed()
*/

#include <stdlib.mqh>

double Slippage_Entry = 5;
double Slippage_Exit  = 10;

color CLR_Buy = Blue;
color CLR_Sell = Red;
color CLR_Exit = Gold;

//---- variables trading
int nActivOrders[30], ActivOrderTyp[30];
double OpenValueOrder[30], StopLossOrder[30], OpenTimeOrder[30];
double ProfitOrder[30];
double nLots;

//####################################################################
//+------------------------------------------------------------------+
//|   Get Activ Orders
//+------------------------------------------------------------------+
void Get_ActivOrder(int iMagicNumber)
{  
   int total = OrdersTotal();   
   for (int is=1; is<=NumberSymbols; is++) { 
      string sSymbol    = Pair[is];
      nActivOrders[is]  = 0;
      ProfitOrder[is]   = 0;
      ActivOrderTyp[is] = -1;

      //---- Order 1
      for(int i=0; i<total; i++) 
      {
         if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {      
            if (OrderSymbol()      == sSymbol && 
                OrderCloseTime()   == 0 &&
                OrderMagicNumber() == iMagicNumber)
            {
               OpenValueOrder[is] = OrderOpenPrice();
               StopLossOrder[is]  = OrderStopLoss();
               ProfitOrder[is]    = OrderProfit();
               OpenTimeOrder[is]  = OrderOpenTime();
               nActivOrders[is]++;                
               ActivOrderTyp[is] = OrderType();
               
               if (mySystemSignal[is][Column_SignalEvaluation_MTF][3] == 0) {
                  if (OrderType() == OP_BUY)
                     mySystemSignal[is][Column_SignalEvaluation_MTF][3] =  1; else
                  if (OrderType() == OP_SELL)
                     mySystemSignal[is][Column_SignalEvaluation_MTF][3] = -1;
               }   
               break;
            }
         }      
      }  
   }
   return;
}


//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//    Send Order BuyMarket
//+------------------------------------------------------------------+
void SendOrder_BuyMarket(string sSymbol, double nLots, int iMagicNumber, string stComment)
{
   double dPrice = MarketInfo(sSymbol, MODE_ASK);
   string sComment = sSymbol + " -buy- " + stComment;
   if (SendOrders_IsAllowed() == true) {
      int ticket1=OrderSend(sSymbol,OP_BUY,nLots,dPrice,Slippage_Entry,0,0, sComment, iMagicNumber,0,CLR_Buy);      
   }
   if(ticket1<0) 
      Send_ErrorDescription(sSymbol, " BUY " + stComment);
   return;
}

//+------------------------------------------------------------------+
//    Send Order SellMarket
//+------------------------------------------------------------------+
void SendOrder_SellMarket(string sSymbol, double nLots, int iMagicNumber, string stComment)
{
   double dPrice = MarketInfo(sSymbol, MODE_BID);
   string sComment = sSymbol + " -sell- " + stComment;
   if (SendOrders_IsAllowed() == true) {
      int ticket1 = OrderSend(sSymbol,OP_SELL,nLots,dPrice,Slippage_Entry,0,0, sComment, iMagicNumber,0,CLR_Sell);
   }
   if(ticket1<0)
      Send_ErrorDescription(sSymbol, " SELL " + stComment); 
   return;   
}


//+------------------------------------------------------------------+
//    Send Orders Is Allowed
//+------------------------------------------------------------------+
bool SendOrders_IsAllowed()
{
return(true);

   bool SendOrderIsOK = false;
   while (SendOrderIsOK == true) {      
      if (IsTradeAllowed()     == true && 
          IsTradeContextBusy() == false) // &&
          //RefreshRates()       == true) 
         SendOrderIsOK = true;
      else
         Sleep(300);   
   }
   return(SendOrderIsOK);
}

//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//| Close Activ Order
//+------------------------------------------------------------------+
void Close_ActivOrder(string sSymbol, int iMagicNumber)
{
   bool CloseResult;  

   int total=OrdersTotal();

   for(int i=0; i<total; i++) {
      if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
         if (OrderSymbol()      == sSymbol && 
             OrderCloseTime()   == 0 &&
             OrderMagicNumber() == iMagicNumber) {
               
            if (OrderType() == OP_BUY) {
               if (SendOrders_IsAllowed() == true) {
                  double dBid = MarketInfo(sSymbol, MODE_BID);
                  CloseResult = OrderClose(OrderTicket(), OrderLots(), dBid, Slippage_Exit, CLR_Exit);
               }   
               if (CloseResult == false) Send_ErrorDescription(OrderSymbol(), " Buy-Exit " + dBid);
               break;
            } else
               
            if (OrderType() == OP_SELL) {
               if (SendOrders_IsAllowed() == true) {
                  double dAsk = MarketInfo(sSymbol, MODE_ASK);
                  CloseResult = OrderClose(OrderTicket(), OrderLots(), dAsk, Slippage_Exit, CLR_Exit);
               }   
               if (CloseResult == false) Send_ErrorDescription(OrderSymbol(), " Sell-Exit " + dAsk);
               break;
            }   
         }
      }
   }
   return;
}

//####################################################################
//+------------------------------------------------------------------+
//    Send_ErrorDescription
//+------------------------------------------------------------------+
void Send_ErrorDescription(string sSymbol, string sText)
{
   int err=GetLastError(); 
   if (sText != "")
   string st = sSymbol + " error(" + err + "): " + ErrorDescription(err)+ " -> " + sText; else
          st = sSymbol + " error(" + err + "): " + ErrorDescription(err);
   Alert(st);
   return;
}


//####################################################################
//+------------------------------------------------------------------+
//|   Get Lots Number
//+------------------------------------------------------------------+
double Get_LotsNumber()
{
   double newLots = Min_Lots;
   return(newLots);   
}



//####################################################################
//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//|   Modify Long StopLoss
//+------------------------------------------------------------------+
bool Modify_LongStopLoss(string sSymbol, int iMagicNumber, double NewStopLoss)
{  
   bool modify = false; 
   int iDigits = MarketInfo(sSymbol,MODE_DIGITS);
   double dSpace = MarketInfo(sSymbol,MODE_STOPLEVEL)*MarketInfo(sSymbol,MODE_POINT);
   double dBid   = MarketInfo(sSymbol, MODE_BID);
   double dStopLoss = NormalizeDouble(NewStopLoss, iDigits);  
   
   if (dStopLoss >= dBid - dSpace) return(false);
   
   int total=OrdersTotal();
   
   for(int i=0; i<total; i++) {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      
         if (OrderSymbol()      == sSymbol && 
             OrderType()        == OP_BUY &&
             OrderCloseTime()   == 0 && 
             OrderMagicNumber() == iMagicNumber) 
         {
            //while (modify == false) { 
               if (SendOrders_IsAllowed() == true)
                  modify = OrderModify(OrderTicket(), OrderOpenPrice(), dStopLoss, 0, 0, CLR_NONE);
               if (modify == false) {
                  string sModify = " Buy ";                  
                  string sPrice  = ", SL= " + DoubleToStr(dStopLoss, iDigits);
                  Send_ErrorDescription(sSymbol, " Modify SL" + sModify + sPrice);
               }               
            //}
            break;
         }
      }
   }
   return(modify);
}


//+------------------------------------------------------------------+
//|   Modify Short StopLoss
//+------------------------------------------------------------------+
bool Modify_ShortStopLoss(string sSymbol, int iMagicNumber, double NewStopLoss)
{
   bool modify = false; 
   int iDigits = MarketInfo(sSymbol,MODE_DIGITS);
   double dSpace = MarketInfo(sSymbol,MODE_STOPLEVEL)*MarketInfo(sSymbol,MODE_POINT);
   double dAsk   = MarketInfo(sSymbol, MODE_ASK);
   double dStopLoss = NormalizeDouble(NewStopLoss, iDigits);   
   
   if (dStopLoss <= dAsk + dSpace) return(false);
   
   int total=OrdersTotal();
   
   for(int i=0; i<total; i++) {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      
         if (OrderSymbol()      == sSymbol && 
             OrderType()        == OP_SELL &&
             OrderCloseTime()   == 0 && 
             OrderMagicNumber() == iMagicNumber) 
         {   
            //while (modify == false) {           
               if (SendOrders_IsAllowed() == true)
                  modify = OrderModify(OrderTicket(), OrderOpenPrice(), dStopLoss, 0, 0, CLR_NONE);            
               if (modify == false) {
                  string sModify = " Sell";
                  string sPrice  = ", SL= " +DoubleToStr(dStopLoss, iDigits);
                  Send_ErrorDescription(sSymbol, " Modify SL" + sModify + sPrice);
               }
            //}     
            break;            
         }
      }
   }
   return(modify);
}

