//+------------------------------------------------------------------+
//|                                      BasketTR_TradingInclude.mqh |
//|                                             2009.06.04 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2009.06.04 SwingMan"
#property link      ""

/* PROCEDURES:
void Check_SendNewOrder()
void Set_InitialStopLoss()
double Get_InitialStopLossValue(int is, string sSymbol, int iOrderType)
void Display_TradingOrders()
void Display_EntrySLValues(int ir, int is, string sSymbol, int iOrderType, double dEntryValue, double dStopLossValue)
*/


//####################################################################
//+------------------------------------------------------------------+
//    Check Send NewOrder
//+------------------------------------------------------------------+
void Check_SendNewOrder()
{  
int iPeriod = iTimeFrame[Column_SignalEvaluation_MTF];
bool bEntryInNewTrend;
   
   for (int ir=1; ir<=NumberSymbols; ir++) { 
      //---- only first N ranks
      if (ir <= nMaxEntryRanking) {
         int is = myRanking[ir][colCombiTFs][0];
         string sSymbol = Pair[is];
         
         //==========================================================
         //---- avoid multiple trades!
         if (nActivOrders[is] == 0) {
            
            //---- avoid reentry in trend after an exit!
            if (Reentry_AfterTrendExit == false) {
               //---- check long reentry
               if (TradingSignal[is] == 1) {
                  bEntryInNewTrend = mySystemSignal[is][Column_SignalEvaluation_MTF][3] <= 0; //-- previous trend was short
               }
               else
               if (TradingSignal[is] == -1) {
                  bEntryInNewTrend = mySystemSignal[is][Column_SignalEvaluation_MTF][3] >= 0; //-- previous trend was long
               }
            }            
            
            //---- entry in a new trend is OK 
            if (bEntryInNewTrend == true) 
            {
               int iDigits    = PairDigits[is];
               double dPoint  = PairPoint[is]; //MarketInfos(sSymbol, MODE_POINT)*MathPow(10,iDigits%2);
               double dOffset_EntryPoints = Offset_EntryPoints * dPoint;
      
               nLots = Get_LotsNumber();
               double dHigh1 = iHigh( sSymbol, iPeriod, 1);
               double dLow1  = iLow(  sSymbol, iPeriod, 1);
               //---- long
               if (TradingSignal[is] == 1) {
                  if (MarketInfo(sSymbol,MODE_BID) >= dHigh1 + dOffset_EntryPoints) { //-- 1P above High1
                     StopLossTrade[is]   = 0;
                     InitialStopLoss[is] = 0;
                     BreakEvenStop[is]   = 0;
                     StrongStopLoss[is]  = 0;
                     SendOrder_BuyMarket(sSymbol, nLots, MagicNumber, EAComment+ir+"]");     //-- BUY at market
                     mySystemSignal[is][Column_SignalEvaluation_MTF][3] = 1; //-- long trend; avoid reentrys in trend
                     mySystemSignal[is][Column_SignalEvaluation_MTF][2] = 0;
                  }   
               }
               else
               //---- short
               if (TradingSignal[is] == -1) {
                  if (MarketInfo(sSymbol,MODE_BID) <= dLow1 - dOffset_EntryPoints) {  //-- 1P below Low1
                     StopLossTrade[is]   = 0;
                     InitialStopLoss[is] = 0;
                     BreakEvenStop[is]   = 0;
                     StrongStopLoss[is]  = 0;
                     SendOrder_SellMarket(sSymbol, nLots, MagicNumber, EAComment+ir+"]");    //-- SELL at Market
                     mySystemSignal[is][Column_SignalEvaluation_MTF][3] = -1; //-- short trend; avoid reentrys in trend
                     mySystemSignal[is][Column_SignalEvaluation_MTF][2] =  0;
                  }   
               }
            }
         }
      }
   }
}

//####################################################################
//+------------------------------------------------------------------+
//    Get Entry Price Value
//+------------------------------------------------------------------+
double Get_EntryPriceValue(int is, string sSymbol, int iOrderType)
{
   int col = Column_SignalEvaluation_MTF;
   int iPeriod = iTimeFrame[col];
   
   double dSpread = PairSpread[is];
   double dSignalOffset = Pair_HiLoOffset[is];
   
   //---- long
   if (iOrderType == 1) {
      return(iHigh(sSymbol, iPeriod, 1) + dSpread + dSignalOffset);
   }
   else
   
   //---- short
   if (iOrderType == -1) {
      return(iLow(sSymbol, iPeriod, 1) - dSignalOffset);
   }
}


//####################################################################
//+------------------------------------------------------------------+
//    Set Initial StopLoss
//+------------------------------------------------------------------+
void Set_InitialStopLoss()
{
   for (int ir=1; ir<=NumberSymbols; ir++) { 
      int is = myRanking[ir][colCombiTFs][0];
      string sSymbol = Pair[is];
                  
      //---- get trading direction
      if (TradingSignal[is]==0) {
         if (ActivOrderTyp[is] == OP_BUY)  TradingSignal[is]=1; else
         if (ActivOrderTyp[is] == OP_SELL) TradingSignal[is]=-1;
      }
                  
      //---- for activ trade, the Initial StopLoss is not setted
      if (nActivOrders[is] != 0 && InitialStopLoss[is] == 0) {
         Get_InitialStopLossOrder(is, sSymbol, TradingSignal[is]);
      }  
   }
   return;
}

//####################################################################
//+------------------------------------------------------------------+
//    Get Initial StopLoss Order; StopLosses are Null
//+------------------------------------------------------------------+
void Get_InitialStopLossOrder(int is, string sSymbol, int iOrderType)
{
double dInitialStopLoss;
int col = Column_SignalEvaluation_MTF;
int iPeriod = iTimeFrame[col];
int shift   = 0;
double maxHigh, minLow;
int i;
double dBreakEven_Excurs, dBreakEven_Value;
double dHiloExcurs, dStrongStopLoss;
double dATR    = myHighLowDistance[is][col];  // HiLo distance
double dSpread = PairSpread[is];
double dSignalOffset = Pair_HiLoOffset[is];   
double dPoint  = PairPoint[is];   
double dBid = MarketInfo(sSymbol,MODE_BID);
double dAsk = MarketInfo(sSymbol,MODE_ASK); 
   
   if (nActivOrders[is] != 0) {
      shift = iBarShift(sSymbol,iPeriod,OpenTimeOrder[is]);   
   }

   //================================================================
   //---- 1. initial stop loss = Hilo or ATR
   if (InitialStopLoss[is] == 0) {
      //---- long
      if (iOrderType == 1) {
         double dLow0   = iLow(  sSymbol, iPeriod, shift);
         double dLow1   = iLow(  sSymbol, iPeriod, shift+1);  
         double LongStop_Lows   = MathMin(dLow0, dLow1)   - (dSignalOffset);
         double LongStop_ATR    = OpenValueOrder[is] - (dATR * Factor_InitialStopLoss);
         dInitialStopLoss = MathMax(LongStop_Lows, LongStop_ATR);
      }
      else   
   
      //---- short
      if (iOrderType == -1) {
         double dHigh0  = iHigh( sSymbol, iPeriod, shift);
         double dHigh1  = iHigh( sSymbol, iPeriod, shift+1);
         double ShortStop_Highs = MathMax(dHigh0, dHigh1) + (dSignalOffset + dSpread);
         double ShortStop_ATR   = OpenValueOrder[is] + (dATR * Factor_InitialStopLoss + dSpread);
         dInitialStopLoss = MathMin(ShortStop_Highs, ShortStop_ATR);
      }
      dInitialStopLoss = NormalizeDouble(dInitialStopLoss, PairDigits[is]);
      InitialStopLoss[is] = dInitialStopLoss;
      StopLossTrade[is]   = dInitialStopLoss;
   }
   
   
   //================================================================
   //---- 2. initial break even      
   if (BreakEvenStop[is] == 0) {
      //---- long
      if (iOrderType == 1) {
         maxHigh = iHigh( sSymbol, iPeriod, shift);
         for (i=shift; i>=0;  i--)
            maxHigh = MathMax(maxHigh, iHigh( sSymbol, iPeriod, i));
         dBreakEven_Excurs = NormalizeDouble(OpenValueOrder[is] + dATR * Factor_BreakEven, PairDigits[is]);   
         dBreakEven_Value  = NormalizeDouble(OpenValueOrder[is] + BreakEven_Points*dPoint, PairDigits[is]);
         if (maxHigh >= dBreakEven_Excurs && 
             ProfitOrder[is] > 0 && 
             dBid > dBreakEven_Value + dSpread) 
         {    
            BreakEvenStop[is] = dBreakEven_Value;
         }   
//if (sSymbol=="GBPCAD")
//Comment("\n\n\n",shift,"  ",maxHigh,"  ",dBreakEven_Excurs,"  ",dBreakEven_Value,"  ",BreakEvenStop[is]);
      }
      else   
      
      //---- short
      if (iOrderType == -1) {
         minLow = iLow( sSymbol, iPeriod, shift);
         for (i=shift; i>=0;  i--)
            minLow = MathMin(minLow, iLow( sSymbol, iPeriod, i));
         dBreakEven_Excurs = NormalizeDouble(OpenValueOrder[is] - dATR * Factor_BreakEven, PairDigits[is]);
         dBreakEven_Value  = NormalizeDouble(OpenValueOrder[is] - BreakEven_Points*dPoint, PairDigits[is]);
         if (minLow <= dBreakEven_Excurs && 
             ProfitOrder[is] > 0 && 
             dAsk < dBreakEven_Value - dSpread)
         {    
            BreakEvenStop[is] = dBreakEven_Value;
         }   
      }
   }
   
   
   //================================================================
   //---- 3. initial Strong SL   
   if (StrongStopLoss[is] == 0) {
      //---- long
      if (iOrderType == 1) {
         maxHigh = iHigh( sSymbol, iPeriod, shift);
         for (i=shift; i>=0;  i--)
            maxHigh = MathMax(maxHigh, iHigh( sSymbol, iPeriod, i));
         dHiloExcurs = myHighLowTrend[is][col][2] + dATR * Factor_HiloTrail;
         dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
         dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][2], PairDigits[is]);
         if (BreakEvenStop[is] > 0 &&
             dStrongStopLoss > OpenValueOrder[is] &&
             maxHigh >= dHiloExcurs && 
             ProfitOrder[is] > 0 && 
             dBid > dStrongStopLoss + dSpread) 
         {    
            StrongStopLoss[is] = dStrongStopLoss;
         }   
      }
      else   
      
      //---- short
      if (iOrderType == -1) {
         minLow = iLow( sSymbol, iPeriod, shift);
         for (i=shift; i>=0;  i--)
            minLow = MathMin(minLow, iLow( sSymbol, iPeriod, i));            
         dHiloExcurs = myHighLowTrend[is][col][3] - dATR * Factor_HiloTrail;
         dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
         dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][3], PairDigits[is]);
         if (BreakEvenStop[is] > 0 &&
             dStrongStopLoss < OpenValueOrder[is] &&
             minLow <= dHiloExcurs && 
             ProfitOrder[is] > 0 &&
             dAsk < dStrongStopLoss - dSpread)             
         {    
            StrongStopLoss[is] = dStrongStopLoss;
         }   
      }
   }
}

//+------------------------------------------------------------------+
//    Get Initial StopLoss Value
//+------------------------------------------------------------------+
double Get_InitialStopLossValue(int is, string sSymbol, int iOrderType, double dEntryValue)
{
   double dInitialStopLoss;
   int col = Column_SignalEvaluation_MTF;
   int iPeriod = iTimeFrame[col];
   int shift   = 0;

   double dATR    = myHighLowDistance[is][col]; // HiLo distance
   double dSpread = PairSpread[is];
   double dSignalOffset = Pair_HiLoOffset[is];
   
   //================================================================
   //---- 1. initial stop loss = Hilo or ATR
   //---- long
   if (iOrderType == 1) {
      double dLow0   = iLow(  sSymbol, iPeriod, shift);
      double dLow1   = iLow(  sSymbol, iPeriod, shift+1);  
      double LongStop_Lows   = MathMin(dLow0, dLow1)   - (dSignalOffset);
      double LongStop_ATR    = dEntryValue - (dATR * Factor_InitialStopLoss);
      dInitialStopLoss = MathMax(LongStop_Lows, LongStop_ATR);
   }
   else   
   //---- short
   if (iOrderType == -1) {
      double dHigh0  = iHigh( sSymbol, iPeriod, shift);
      double dHigh1  = iHigh( sSymbol, iPeriod, shift+1);
      double ShortStop_Highs = MathMax(dHigh0, dHigh1) + (dSignalOffset + dSpread);
      double ShortStop_ATR   = dEntryValue + (dATR * Factor_InitialStopLoss + dSpread);
      dInitialStopLoss = MathMin(ShortStop_Highs, ShortStop_ATR);
   }
   return(NormalizeDouble(dInitialStopLoss, PairDigits[is]));
}

//####################################################################
//+------------------------------------------------------------------+
//    Send Order StopLoss
//+------------------------------------------------------------------+
void Send_OrderStopLoss(int is, string sSymbol)
{
   double dBid = MarketInfo(sSymbol,MODE_BID);
   double dAsk = MarketInfo(sSymbol,MODE_ASK);
   double dStrongStopLoss = NormalizeDouble(StrongStopLoss[is], PairDigits[is]);
   double dBreakEven_Value= NormalizeDouble(BreakEvenStop[is],  PairDigits[is]);
   double dSpread = PairSpread[is];
   
if (sSymbol=="GBPCAD")   
Comment("\n\n\n",dStrongStopLoss,"  ",dBreakEven_Value,"  ",PairDigits[is],"  ",dSpread);
   //================================================================
   //---- 1. check sending Strong SL
   if (dStrongStopLoss > 0 && dBreakEven_Value > 0 && ProfitOrder[is] > 0)
   {
      //---- long trade
      if (ActivOrderTyp[is] == OP_BUY) {   
         if (dStrongStopLoss > OpenValueOrder[is] &&
             dStrongStopLoss > dBreakEven_Value &&
             dBid > dStrongStopLoss + dSpread) 
         {
            Modify_LongStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
            return;
         }
      }
      else
   
      //---- short trade
      if (ActivOrderTyp[is] == OP_SELL) {
         if (dStrongStopLoss < OpenValueOrder[is] &&
             dStrongStopLoss < dBreakEven_Value &&
             dAsk < dStrongStopLoss - dSpread) 
         {
            Modify_ShortStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
            return;     
         }
      }
   }
   
   //================================================================
   //---- 2. check sending BreakEven SL
   if (dBreakEven_Value > 0 && ProfitOrder[is] > 0)
   {
      //---- long trade
      if (ActivOrderTyp[is] == OP_BUY) {   
         if (dBreakEven_Value > OpenValueOrder[is] &&
             dBid > dBreakEven_Value + dSpread) 
         {
            Modify_LongStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
            return;
         }
      }
      else
   
      //---- short trade
      if (ActivOrderTyp[is] == OP_SELL) {
         if (dBreakEven_Value < OpenValueOrder[is] &&
             dAsk < dBreakEven_Value - dSpread) 
         {
            Modify_ShortStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
            return;     
         }
      }
   }
}

//+------------------------------------------------------------------+
//    Set New StopLoss
//+------------------------------------------------------------------+
void Set_NewStopLoss()
{
int col = Column_SignalEvaluation_MTF;
int iPeriod = iTimeFrame[col];
double dBreakEven_Excurs, dHiloExcurs, dStrongStopLoss, dBreakEven_Value;
      
   for (int ir=1; ir<=NumberSymbols; ir++) { 
      int is = myRanking[ir][colCombiTFs][0];
      string sSymbol = Pair[is];
      
      if (nActivOrders[is] != 0) {         
         
         //==========================================================
         //---- A. StopLossOrder is not setted
         //----    Set Strong SL, or BreakEven
         if (StopLossOrder[is] == 0) {   
            Send_OrderStopLoss(is, sSymbol);
         } 
         else
         
         //==========================================================
         //---- B. StopLossOrder is setted, ProfitOrder is > 0
         //----    Set Strong SL as trailing stop of Hilo lines
         if (StopLossOrder[is] > 0 && ProfitOrder[is] > 0) {   
            double dATR    = myHighLowDistance[is][col]; // 1/2 HiLo distance      
            double dSpread = PairSpread[is];
            double dPoint  = PairPoint[is];
            double dBid = MarketInfo(sSymbol,MODE_BID);
            double dAsk = MarketInfo(sSymbol,MODE_ASK);
            
            //---- long trade
            if (ActivOrderTyp[is] == OP_BUY) {
               dHiloExcurs = myHighLowTrend[is][col][2] + dATR * Factor_HiloTrail;
               dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
               dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][2], PairDigits[is]);
               if (dStrongStopLoss > StopLossOrder[is] &&
                   dBid > dStrongStopLoss + dSpread) 
               {
                   Modify_LongStopLoss(sSymbol, MagicNumber, dStrongStopLoss);        
               }
            }
            else
            
            //---- short trade
            if (ActivOrderTyp[is] == OP_SELL) {
               dHiloExcurs = myHighLowTrend[is][col][3] - dATR * Factor_HiloTrail;
               dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
               dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][3], PairDigits[is]);
               if (dStrongStopLoss < StopLossOrder[is] &&
                   dAsk < dStrongStopLoss - dSpread) 
               {
                  Modify_ShortStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
               }
            }
         }
      }
//if (dStrongStopLoss > 0 && dBreakEven_Value > 0 && ProfitOrder[is] > 0)      
      /*            
      //=============================================================
      //---- 1. strong stop loss is not setted
      if (nActivOrders[is] != 0 && StrongStopLoss[is] == 0) {
         if (StopLossOrder[is] > 0) StrongStopLoss[is] = StopLossOrder[is]; else
         if (BreakEvenStop[is] > 0) StrongStopLoss[is] = BreakEvenStop[is]; else
         //---- set new strong SL
         {
            //---- long trade
            if (ActivOrderTyp[is] == OP_BUY) {
               dHiloExcurs = myHighLowTrend[is][col][2] + dATR * Factor_HiloTrail;
               if (dBid >= dHiloExcurs && ProfitOrder[is] > 0) {
                  dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][2], PairDigits[is]);                  
                  StrongStopLoss[is] = dStrongStopLoss; // Stop=Highs blue line                  
                  StopLossTrade[is]  = dStrongStopLoss;
                  if (dStrongStopLoss != StopLossOrder[is]) {
                     Modify_LongStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
                  }
               }
            }
            else
            //---- short trade
            if (ActivOrderTyp[is] == OP_SELL) {
               //---- 2.2.2 set Strong StopLoss; Ask < Lows - ATR
               dHiloExcurs = myHighLowTrend[is][col][3] - dATR * Factor_HiloTrail;
               if (dAsk <= dHiloExcurs && ProfitOrder[is] > 0) {
                  dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][3], PairDigits[is]);
                  StrongStopLoss[is] = dStrongStopLoss; // Stop=Lows red line
                  StopLossTrade[is]  = dStrongStopLoss;            
                  if (dStrongStopLoss != StopLossOrder[is]) {
                     Modify_ShortStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
                  }            
               }      
            }
         }
      }
      else*/
/*      
      //================================================================
      //---- 2. Initial StopLoss is setted, check break even and strong SL
      if (nActivOrders[is] != 0 && StopLossTrade[is] != 0) {         
         //---- long trade
         if (ActivOrderTyp[is] == OP_BUY) 
         {  
            //---- 2.1.1 set break even long
            dBreakEven_Excurs = NormalizeDouble(OpenValueOrder[is] + dATR * Factor_BreakEven, PairDigits[is]);
            dBreakEven_Value  = NormalizeDouble(OpenValueOrder[is] + BreakEven_Points*dPoint, PairDigits[is]);
if (sSymbol=="GBPUSD")
Comment("\n\n\n",dBreakEven_Value,"  ",StopLossOrder[is]);
            if (dBid >= dBreakEven_Excurs && ProfitOrder[is] > 0) {
               if (StopLossOrder[is] == 0) 
               {
                  BreakEvenStop[is] = dBreakEven_Value;
                  StrongStopLoss[is]= dBreakEven_Value;
                  StopLossTrade[is] = dBreakEven_Value;
                  Modify_LongStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
               }
               else
               if (StopLossOrder[is] != dBreakEven_Value) 
               {
                  BreakEvenStop[is] = dBreakEven_Value;
                  StrongStopLoss[is]= dBreakEven_Value;
                  StopLossTrade[is] = dBreakEven_Value;
                  Modify_LongStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
               }
            }   
            //---- 2.1.2 set Strong StopLoss; Bid > Highs + ATR
            dHiloExcurs = myHighLowTrend[is][col][2] + dATR * Factor_HiloTrail;
            dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
            if (dBid >= dHiloExcurs && ProfitOrder[is] > 0) {
               dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][2], PairDigits[is]);
               StrongStopLoss[is] = dStrongStopLoss; // Stop=Highs blue line              
               StopLossTrade[is]  = dStrongStopLoss;
               if (dStrongStopLoss != StopLossOrder[is] && dStrongStopLoss > BreakEvenStop[is]) {
                  Modify_LongStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
               }
            }   
//if (sSymbol=="AUDCAD")
//Comment("\n\n\n", nActivOrders[is], "  ",InitialStopLoss[is],"  ",StopLossTrade[is],
//                  "  ",BreakEvenStop[is],"  ", StrongStopLoss[is],"  Profit=",ProfitOrder[is]);
         }
         else
   
         //================================================================
         //---- short trade 
         if (ActivOrderTyp[is] == OP_SELL) 
         {
            //---- 2.2.1 set break even short
            dBreakEven_Excurs = NormalizeDouble(OpenValueOrder[is] - dATR * Factor_BreakEven, PairDigits[is]);
            dBreakEven_Value  = NormalizeDouble(OpenValueOrder[is] - BreakEven_Points*dPoint, PairDigits[is]);
            if (dAsk <= dBreakEven_Excurs && ProfitOrder[is] > 0) {
               if (StopLossOrder[is] == 0) {
                  BreakEvenStop[is] = dBreakEven_Value;
                  StrongStopLoss[is]= dBreakEven_Value;
                  StopLossTrade[is] = dBreakEven_Value;
                  Modify_ShortStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
               }
               else
               if (StopLossOrder[is] > dBreakEven_Value) {
                  BreakEvenStop[is] = dBreakEven_Value;
                  StrongStopLoss[is]= dBreakEven_Value;
                  StopLossTrade[is] = dBreakEven_Value;
                  Modify_ShortStopLoss(sSymbol, MagicNumber, dBreakEven_Value);
               }
            }   
            //---- 2.2.2 set Strong StopLoss; Ask < Lows - ATR
            dHiloExcurs = myHighLowTrend[is][col][3] - dATR * Factor_HiloTrail;
            dHiloExcurs = NormalizeDouble(dHiloExcurs, PairDigits[is]);
            if (dAsk <= dHiloExcurs && ProfitOrder[is] > 0) {
               dStrongStopLoss = NormalizeDouble(myHighLowTrend[is][col][3], PairDigits[is]);
               StrongStopLoss[is] = dStrongStopLoss; // Stop=Lows red line
               StopLossTrade[is]  = dStrongStopLoss;            
               if (dStrongStopLoss != StopLossOrder[is] && dStrongStopLoss < BreakEvenStop[is]) {
                  Modify_ShortStopLoss(sSymbol, MagicNumber, dStrongStopLoss);
               }            
            }
         }       
      }*/
   }
   return;
}

//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//    1. Display Trading Orders
//+------------------------------------------------------------------+
void Display_TradingOrders()
{
int ir, is;
string sSymbol, sNameArrow, sNameWin;
string sValue;
color dColor;
int xx, iArrow, iHght, iHeight = 10;
double dValue, sumWinLoss;
int iOrderType;
double dEntryValue, dStopLossValue;
/*
//#### TEST ####
bool TestOK = false;
if (TestOK == true) {
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];
      
      //-- long
      if (sSymbol == "CADJPY") {
         nActivOrders[is] = 1;
         ActivOrderTyp[is]=OP_BUY;
         OpenValueOrder[is] = 85.10;
         StopLossTrade[is]  = 84.00;
         ProfitOrder[is]    = 12.13;
      }
      if (sSymbol == "EURJPY") {
         nActivOrders[is] = 1;
         ActivOrderTyp[is]=OP_BUY;
         OpenValueOrder[is] = 137.25;
         StopLossTrade[is]  = 136.75;
         ProfitOrder[is]    = 11.20;
      }
      if (sSymbol == "CHFJPY") {
         nActivOrders[is] = 1;
         ActivOrderTyp[is]=OP_BUY;
         OpenValueOrder[is] = 90.20;
         StopLossTrade[is]  = 89.75;
         ProfitOrder[is]    = -10.40;
      }
      //-- short
      if (sSymbol == "GBPNZD") {
         nActivOrders[is] = 1;
         ActivOrderTyp[is]=OP_SELL;
         OpenValueOrder[is] = 2.5295;
         StopLossTrade[is]  = 2.5175;
         ProfitOrder[is]    = 20.22;
      }
   }
}*/

//=============================================
   sumWinLoss=0;
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];
      
      //---- names
      sNameArrow = "_TradeArrow_" + sSymbol;
//      sNameEntry = "_TradeEntry_" + sSymbol;
//      sNameSL    = "_TradeSL_" + sSymbol;
      sNameWin   = "_TradeWin_" + sSymbol;
      
      //-- trading is activ
      if (nActivOrders[is] != 0) {
         //==========================================================
         //---- 1. Trading Arrows
         xx = xCol[colTrade_Arrow];
         //---- LONG trade
         if (ActivOrderTyp[is] == OP_BUY) {
            iArrow = 233;
            dColor = colorPlusSignal;
            iOrderType = 1;
         }
         else
         //---- SHORT trade
         if (ActivOrderTyp[is] == OP_SELL) {
            iArrow = 234;
            dColor = colorMinusSignal;
            iOrderType = -1;
         }
         SetArrowObject(sNameArrow, iArrow, iHeight, dColor, xx, yRow[ir]);
         
         //==========================================================
         //---- 2. Entry and StopLoss values
         dEntryValue    = OpenValueOrder[is];        

         if (StrongStopLoss[is]  > 0) dStopLossValue = StrongStopLoss[is]; else
         if (BreakEvenStop[is]   > 0) dStopLossValue = BreakEvenStop[is];  else
         if (InitialStopLoss[is] > 0) dStopLossValue = InitialStopLoss[is];else
                                      dStopLossValue = StopLossTrade[is];
//if (sSymbol=="EURAUD")
//Comment("\n\n\n", nActivOrders[is], "  ",InitialStopLoss[is],"  ",StopLossTrade[is],
//                  "  ",BreakEvenStop[is],"  ", StrongStopLoss[is]);

         Display_EntrySLValues(ir, is, sSymbol, iOrderType, dEntryValue, dStopLossValue);         
         
         //==========================================================
         //---- 3. Win/Loss Values
         xx = xCol[colTrade_Win];
         dValue = ProfitOrder[is];
         sumWinLoss = sumWinLoss + dValue;
         sValue = DoubleToStr(dValue,2);         
         if (dValue >= 0) dColor = colorPlusSignal; else dColor = colorMinusSignal;
         /*~not so good...~int Len = StringLen(sValue);
         switch (Len) {
            case 3: sValue = "  " + sValue; break;
            case 4: sValue = " " + sValue; break;
            case 5: sValue = "" + sValue; break;
            case 6: sValue = "" + sValue; break;
         }         
         SetLabelObjectValues(sNameWin, sValue, dColor, xx, yRow[ir]);*/
         SetLabelObject(sNameWin, sValue, dColor, xx, yRow[ir]);
      }
      else
      
      //---- trading is not activ
      //---- delete objects
      {
         if (DisplaySignal[is] == 0) {
            iArrow = 159; dColor = colorTitle; iHght = 2;  //-- neutral arrow = point
            xx = xCol[colTrade_Arrow];
            SetArrowObject(sNameArrow, iArrow, iHght, dColor, xx, yRow[ir]);
         }
         //DeleteObject(sNameEntry);
         //DeleteObject(sNameSL);
         DeleteObject(sNameWin);
         //Display_EntrySLValues(ir, is, sSymbol, 0,0,0);
      }
   }
   //---- sum Win/Loss
   if (sumWinLoss >= 0) dColor = colorPlusSignal; else dColor = colorMinusSignal;
   sValue = DoubleToStr(sumWinLoss,2);
   xx = xCol[colTrade_SL];
   SetLabelObject("_TradeSum", "Sum:", colorTitle, xx, yRow[rowTrade_Result]);
   xx = xCol[colTrade_Win];
   SetLabelObject("_TradeSumVal", sValue, dColor, xx, yRow[rowTrade_Result]);
   return;
}

//+------------------------------------------------------------------+
//    2. Display Trading Entry and SL Values
//+------------------------------------------------------------------+
void Display_EntrySLValues(int ir, int is, string sSymbol, int iOrderType, double dEntryValue, double dStopLossValue)
{  
color dColor;   
int iArrow, iHght, iHeight = 10;
int xx = xCol[colTrade_Arrow];

   //---- names
   string sNameArrow = "_TradeArrow_" + sSymbol;
   string sNameEntry = "_TradeEntry_" + sSymbol;
   string sNameSL    = "_TradeSL_" + sSymbol;
   //---- delete
   if (iOrderType == 0 && nActivOrders[is] == 0) {
      DeleteObject(sNameEntry);
      DeleteObject(sNameSL);
      if (EATrading_Activ == true) {
         iArrow = 159; dColor = colorTitle; iHght = 2;  //-- neutral arrow = point      
         SetArrowObject(sNameArrow, iArrow, iHght, dColor, xx, yRow[ir]);
      }
      return;
   } 
   
   //---- initialisations
   int col = Column_SignalEvaluation_MTF;
   int iPeriod = iTimeFrame[col];
   int xxEntry = xCol[colTrade_Entry];
   int xxSL    = xCol[colTrade_SL];
   string sEntryValue    = DoubleToStr(dEntryValue,PairDigits[is]);  
   string sStopLossValue = DoubleToStr(dStopLossValue,PairDigits[is]);
   
   //---- long
   if (iOrderType == 1) {
      dColor = colorPlusIndi; 
      SetLabelObject(sNameEntry, sEntryValue, dColor, xxEntry, yRow[ir]);
      SetLabelObject(sNameSL, sStopLossValue, dColor, xxSL, yRow[ir]);
      if (EATrading_Activ == true) {
         //-- trading is not activ
         if (nActivOrders[is] == 0) {
            iArrow = 217;
            SetArrowObject(sNameArrow, iArrow, iHeight, dColor, xx, yRow[ir]);
         }
      }
   }
   else
   
   //---- short
   if (iOrderType == -1) {   
      dColor = colorMinusIndi; 
      SetLabelObject(sNameEntry, sEntryValue, dColor, xxEntry, yRow[ir]);
      SetLabelObject(sNameSL, sStopLossValue, dColor, xxSL, yRow[ir]);
      if (EATrading_Activ == true) {
         //-- trading is not activ
         if (nActivOrders[is] == 0) {
            iArrow = 218;
            SetArrowObject(sNameArrow, iArrow, iHeight, dColor, xx, yRow[ir]);
         }
      }
   }
   return;
}

//####################################################################
//####################################################################
//+------------------------------------------------------------------+
//    Check Exit Order
//+------------------------------------------------------------------+
void Check_ExitOrder()
{
int col = Column_SignalEvaluation_MTF;
int iPeriod = iTimeFrame[col];
int exitSignal;
string sSymbol;
int ir, is;
double dHigh1, dLow1, dClose1;
double dBid, dAsk, dSpread, dSignalOffset;

   //=================================================================
   //-- 1. exit for change direction of HullTrend(8)
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];
      dBid = MarketInfo(sSymbol,MODE_BID);
      dAsk = MarketInfo(sSymbol,MODE_ASK); 
      dSignalOffset = Pair_HiLoOffset[is];
      dSpread = PairSpread[is];
      double iHullSignal = myHullAvgSignal[is][col][1];
      dHigh1  = iHigh( sSymbol, iPeriod, 1);
      dLow1   = iLow(  sSymbol, iPeriod, 1);      
      exitSignal = 0;

//if (sSymbol=="AUDUSD")
//Comment("\n\n\n","O=",nActivOrders[is],"  P=",ProfitOrder[is],"  B=",dBid,"  L=",dLow1,"  ",dSignalOffset,"  ",iHullSignal);
      
      if (nActivOrders[is] != 0 && ProfitOrder[is] > 0) { // only when profit > 0
         //---- check long exit
         if (ActivOrderTyp[is] == OP_BUY) {
            if (iHullSignal  < 0 && dBid <= dLow1 - dSignalOffset) {
               exitSignal = 1;                
               //Print("==BUY Exit Low=" + DoubleToStr(dLow1,Digits));
            }
         }
         else
         //---- check short exit
         if (ActivOrderTyp[is] == OP_SELL) {
            if (iHullSignal  > 0 && dAsk >= dHigh1 + dSignalOffset + dSpread) {
               exitSignal = -1; 
               //Print("==SELL Exit High=" + DoubleToStr(dHigh1,Digits));
            }         
         }
         
         //-- send exit order for change direction ------------------------
         if (exitSignal != 0) {
            //-- send exit order
            Send_ExitOrder(ir, is, sSymbol, MagicNumber, col);
         }         
      }
   }   

   
   //================================================================
   //================================================================
   //================================================================
   //-- 2. exit for Initial StopLoss (soft exit, when close[1] < virtual SL)
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];
      dBid = MarketInfo(sSymbol,MODE_BID);
      dAsk = MarketInfo(sSymbol,MODE_ASK); 
      dSignalOffset = Pair_HiLoOffset[is];
      dSpread = PairSpread[is];
      dHigh1  = iHigh( sSymbol, iPeriod, 1);
      dLow1   = iLow(  sSymbol, iPeriod, 1);
      dClose1 = iClose(sSymbol, iPeriod, 1);
      exitSignal = 0;
      double dInitialStopLoss = InitialStopLoss[is];
//if (sSymbol=="CADJPY")
//Comment("\n\n\n","C=",dClose1,"  S=",dInitialStopLoss,"  B=",dBid,"  L=",dLow1,"  ",dSignalOffset,"  ",dSpread,"  ");
//Comment("\n\n\n","C=",dClose1,"  S=",dInitialStopLoss,"  A=",dAsk,"  H=",dHigh1,"  ",dSignalOffset,"  ",dSpread,"  ");
//Comment("\n\n\n",StopLossTrade[is],"  ",InitialStopLoss[is],"  ",dSignalOffset,"  ",dSpread,"  ",dClose1);
      if (nActivOrders[is] != 0 && dInitialStopLoss > 0) {
         //---- check long exit
         if (ActivOrderTyp[is] == OP_BUY) {
            if (dClose1  < dInitialStopLoss && dBid <= dLow1 - dSignalOffset) {
               exitSignal = 1; 
               //Print("==BUY Exit Low=" + DoubleToStr(dLow1,Digits));
            }
         }
         else
         //---- check short exit
         if (ActivOrderTyp[is] == OP_SELL) {
            if (dClose1  > dInitialStopLoss && dAsk >= dHigh1 + dSignalOffset + dSpread) {
               exitSignal = -1; 
               //Print("==SELL Exit High=" + DoubleToStr(dHigh1,Digits));
            }         
         }
         
         //-- send exit order for Initial StopLoss ------------------------
         if (exitSignal != 0) {
            //-- send exit order
            Send_ExitOrder(ir, is, sSymbol, MagicNumber, col);
         }         
      }      
   }
   
      
   //================================================================
   //-- 3. forced exit order for Entry - ATR
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];
      
      double dATR = myHighLowDistance[is][col] * Factor_InitialStopLoss; // HiLo distance
      dBid = MarketInfo(sSymbol,MODE_BID);
      dAsk = MarketInfo(sSymbol,MODE_ASK); 
      dSignalOffset = Pair_HiLoOffset[is];
      dSpread = PairSpread[is];
      exitSignal = 0;
//if (sSymbol=="CADJPY")
//Comment("\n\n\n","C=",dClose1,"  Op=",OpenValueOrder[is],"  B=",dBid,"  Off=",dSignalOffset,"  ",dSpread,"  ",dATR);
      if (nActivOrders[is] != 0) {
         //---- check long exit
         if (ActivOrderTyp[is] == OP_BUY) {
            if (dBid <= OpenValueOrder[is] - (dATR + dSignalOffset)) {
               exitSignal = 1; 
               //Print("==BUY Exit Low=" + DoubleToStr(dLow1,Digits));
            }
         }
         else
         //---- check short exit
         if (ActivOrderTyp[is] == OP_SELL) {
            if (dAsk >= OpenValueOrder[is] + (dATR + dSignalOffset + dSpread)) {
               exitSignal = -1; 
               //Print("==SELL Exit High=" + DoubleToStr(dHigh1,Digits));
            }         
         }
         
         //-- send exit order for Entry + ATR  ------------------------
         if (exitSignal != 0) {
            //-- send exit order
            Send_ExitOrder(ir, is, sSymbol, MagicNumber, col);
         }         
      }            
   }
   
   //================================================================
   //-- 4. forced exit order if ranking > 14
   for (ir=1; ir<=NumberSymbols; ir++) { 
      is = myRanking[ir][colCombiTFs][0];
      sSymbol = Pair[is];     

      if (nActivOrders[is] != 0 && ir > nMaxEntryRanking) {
         //-- send exit order
         Send_ExitOrder(ir, is, sSymbol, MagicNumber, col);
      }            
   }   
   return;
}

//-------------------------------------------------------------------
//    Send Exit Order
//-------------------------------------------------------------------
void Send_ExitOrder(int ir, int is, string sSymbol, int iMagicNumber, int col)
{
   //-- send exit order
   Close_ActivOrder(sSymbol, iMagicNumber);
   StopLossTrade[is] = 0;
   
   //-- avoid reentry
   mySystemSignal[is][col][2] = 100;
   
   //-- draw neutral arrow = point
   string sNameArrow = "_TradeArrow_" + sSymbol;
   int iArrow   = 159; 
   color dColor = colorTitle; 
   int iHght    = 2;  
   int xx       = xCol[colTrade_Arrow];
   SetArrowObject(sNameArrow, iArrow, iHght, dColor, xx, yRow[ir]);
}